package client

import (
	"context"
	"fmt"

	apemanager_v2 "git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/apemanager"
	rpcapi "git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/rpc"
	"git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/rpc/client"
	session_v2 "git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/session"
	"git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/signature"
	apemanager_sdk "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/apemanager"
	apistatus "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/client/status"
)

// PrmAPEManagerAddChain groups parameters of APEManagerAddChain operation.
type PrmAPEManagerAddChain struct {
	XHeaders []string

	ChainTarget apemanager_sdk.ChainTarget

	Chain apemanager_sdk.Chain
}

func (prm *PrmAPEManagerAddChain) buildRequest(c *Client) (*apemanager_v2.AddChainRequest, error) {
	if len(prm.XHeaders)%2 != 0 {
		return nil, errorInvalidXHeaders
	}

	req := new(apemanager_v2.AddChainRequest)
	reqBody := new(apemanager_v2.AddChainRequestBody)

	reqBody.SetTarget(prm.ChainTarget.ToV2())
	reqBody.SetChain(prm.Chain.ToV2())

	req.SetBody(reqBody)

	var meta session_v2.RequestMetaHeader
	writeXHeadersToMeta(prm.XHeaders, &meta)

	c.prepareRequest(req, &meta)

	return req, nil
}

type ResAPEManagerAddChain struct {
	statusRes

	// ChainID of set Chain. If Chain does not contain chainID before request, then
	// ChainID is generated.
	ChainID apemanager_sdk.ChainID
}

// APEManagerAddChain sets Chain for ChainTarget.
func (c *Client) APEManagerAddChain(ctx context.Context, prm PrmAPEManagerAddChain) (*ResAPEManagerAddChain, error) {
	req, err := prm.buildRequest(c)
	if err != nil {
		return nil, err
	}

	if err := signature.SignServiceMessage(&c.prm.Key, req); err != nil {
		return nil, fmt.Errorf("sign request: %w", err)
	}

	resp, err := rpcapi.AddChain(&c.c, req, client.WithContext(ctx))
	if err != nil {
		return nil, err
	}

	var res ResAPEManagerAddChain
	res.st, err = c.processResponse(resp)
	if err != nil || !apistatus.IsSuccessful(res.st) {
		return &res, err
	}

	res.ChainID = resp.GetBody().GetChainID()

	return &res, nil
}
