package client

import (
	"context"
	"fmt"

	apemanager_v2 "git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/apemanager"
	rpcapi "git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/rpc"
	"git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/rpc/client"
	session_v2 "git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/session"
	"git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/signature"
	apemanager_sdk "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/apemanager"
	apistatus "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/client/status"
)

// PrmAPEManagerListChains groups parameters of APEManagerListChains operation.
type PrmAPEManagerListChains struct {
	XHeaders []string

	ChainTarget apemanager_sdk.ChainTarget
}

func (prm *PrmAPEManagerListChains) buildRequest(c *Client) (*apemanager_v2.ListChainsRequest, error) {
	if len(prm.XHeaders)%2 != 0 {
		return nil, errorInvalidXHeaders
	}

	req := new(apemanager_v2.ListChainsRequest)
	reqBody := new(apemanager_v2.ListChainsRequestBody)

	reqBody.SetTarget(prm.ChainTarget.ToV2())

	req.SetBody(reqBody)

	var meta session_v2.RequestMetaHeader
	writeXHeadersToMeta(prm.XHeaders, &meta)

	c.prepareRequest(req, &meta)

	return req, nil
}

type ResAPEManagerListChains struct {
	statusRes

	Chains []apemanager_sdk.Chain
}

// APEManagerListChains lists Chains for ChainTarget.
func (c *Client) APEManagerListChains(ctx context.Context, prm PrmAPEManagerListChains) (*ResAPEManagerListChains, error) {
	req, err := prm.buildRequest(c)
	if err != nil {
		return nil, err
	}

	if err := signature.SignServiceMessage(&c.prm.Key, req); err != nil {
		return nil, fmt.Errorf("sign request: %w", err)
	}

	resp, err := rpcapi.ListChains(&c.c, req, client.WithContext(ctx))
	if err != nil {
		return nil, err
	}

	var res ResAPEManagerListChains
	res.st, err = c.processResponse(resp)
	if err != nil || !apistatus.IsSuccessful(res.st) {
		return nil, err
	}

	for _, ch := range resp.GetBody().GetChains() {
		var chSDK apemanager_sdk.Chain
		if err := chSDK.ReadFromV2(ch); err != nil {
			return nil, err
		}
		res.Chains = append(res.Chains, chSDK)
	}

	return &res, nil
}
