package client

import (
	"context"
	"fmt"

	apemanager_v2 "git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/apemanager"
	rpcapi "git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/rpc"
	"git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/rpc/client"
	session_v2 "git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/session"
	"git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/signature"
	apemanager_sdk "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/apemanager"
	apistatus "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/client/status"
)

// PrmAPEManagerRemoveChain groups parameters of APEManagerRemoveChain operation.
type PrmAPEManagerRemoveChain struct {
	XHeaders []string

	ChainTarget apemanager_sdk.ChainTarget

	ChainID apemanager_sdk.ChainID
}

func (prm *PrmAPEManagerRemoveChain) buildRequest(c *Client) (*apemanager_v2.RemoveChainRequest, error) {
	if len(prm.XHeaders)%2 != 0 {
		return nil, errorInvalidXHeaders
	}

	req := new(apemanager_v2.RemoveChainRequest)
	reqBody := new(apemanager_v2.RemoveChainRequestBody)

	reqBody.SetTarget(prm.ChainTarget.ToV2())
	reqBody.SetChainID(prm.ChainID)

	req.SetBody(reqBody)

	var meta session_v2.RequestMetaHeader
	writeXHeadersToMeta(prm.XHeaders, &meta)

	c.prepareRequest(req, &meta)

	return req, nil
}

type ResAPEManagerRemoveChain struct {
	statusRes
}

// APEManagerRemoveChain removes Chain with ChainID defined for ChainTarget.
func (c *Client) APEManagerRemoveChain(ctx context.Context, prm PrmAPEManagerRemoveChain) (*ResAPEManagerRemoveChain, error) {
	req, err := prm.buildRequest(c)
	if err != nil {
		return nil, err
	}

	if err := signature.SignServiceMessage(&c.prm.Key, req); err != nil {
		return nil, fmt.Errorf("sign request: %w", err)
	}

	resp, err := rpcapi.RemoveChain(&c.c, req, client.WithContext(ctx))
	if err != nil {
		return nil, err
	}

	var res ResAPEManagerRemoveChain
	res.st, err = c.processResponse(resp)
	if err != nil || !apistatus.IsSuccessful(res.st) {
		return &res, err
	}

	return &res, nil
}
